"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tailPipeline = exports.togglePausePipeline = exports.detachPipeline = exports.attachPipeline = exports.buildPipeline = exports.processResponse = exports.initPipelines = void 0;
const sp_common_1 = require("@streamdal/protos/protos/sp_common");
const register_js_1 = require("./register.js");
const wasm_js_1 = require("./wasm.js");
const initPipelines = (configs) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, e_1, _b, _c, _d, e_2, _e, _f;
    try {
        if (register_js_1.internal.pipelineInitialized) {
            return;
        }
        console.debug("initializing pipelines");
        const { response } = yield configs.grpcClient.getAttachCommandsByService({
            serviceName: configs.serviceName.toLowerCase(),
        }, { meta: { "auth-token": configs.streamdalToken } });
        try {
            for (var _g = true, _h = __asyncValues(Object.entries(response.wasmModules)), _j; _j = yield _h.next(), _a = _j.done, !_a; _g = true) {
                _c = _j.value;
                _g = false;
                const [k, v] = _c;
                yield (0, wasm_js_1.instantiateWasm)(k, v.bytes);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_g && !_a && (_b = _h.return)) yield _b.call(_h);
            }
            finally { if (e_1) throw e_1.error; }
        }
        try {
            for (var _k = true, _l = __asyncValues(response.active), _m; _m = yield _l.next(), _d = _m.done, !_d; _k = true) {
                _f = _m.value;
                _k = false;
                const command = _f;
                yield (0, exports.processResponse)(command);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (!_k && !_d && (_e = _l.return)) yield _e.call(_l);
            }
            finally { if (e_2) throw e_2.error; }
        }
        register_js_1.internal.pipelineInitialized = true;
    }
    catch (e) {
        console.error("Error initializing pipelines", e);
    }
});
exports.initPipelines = initPipelines;
const processResponse = (response) => __awaiter(void 0, void 0, void 0, function* () {
    if (!response.audience) {
        response.command.oneofKind !== "keepAlive" &&
            console.debug("command response has no audience, ignoring");
        return;
    }
    switch (response.command.oneofKind) {
        case "attachPipeline":
            response.command.attachPipeline.pipeline &&
                (yield (0, exports.attachPipeline)(response.audience, response.command.attachPipeline.pipeline));
            break;
        case "detachPipeline":
            (0, exports.detachPipeline)(response.audience, response.command.detachPipeline.pipelineId);
            break;
        case "pausePipeline":
            (0, exports.togglePausePipeline)(response.audience, response.command.pausePipeline.pipelineId, true);
            break;
        case "resumePipeline":
            (0, exports.togglePausePipeline)(response.audience, response.command.resumePipeline.pipelineId, false);
            break;
        case "tail":
            (0, exports.tailPipeline)(response.audience, response.command.tail);
            break;
    }
});
exports.processResponse = processResponse;
const buildPipeline = (pipeline) => __awaiter(void 0, void 0, void 0, function* () {
    return Object.assign(Object.assign({}, pipeline), { steps: yield Promise.all(pipeline.steps.map((step) => __awaiter(void 0, void 0, void 0, function* () {
            yield (0, wasm_js_1.instantiateWasm)(step.WasmId, step.WasmBytes);
            return Object.assign(Object.assign({}, step), { WasmBytes: undefined });
        }))) });
});
exports.buildPipeline = buildPipeline;
const attachPipeline = (audience, pipeline) => __awaiter(void 0, void 0, void 0, function* () {
    const key = (0, register_js_1.audienceKey)(audience);
    const existing = register_js_1.internal.pipelines.get(key);
    const built = yield (0, exports.buildPipeline)(pipeline);
    register_js_1.internal.pipelines.set(key, existing
        ? existing.set(pipeline.id, built)
        : new Map([[pipeline.id, built]]));
});
exports.attachPipeline = attachPipeline;
const detachPipeline = (audience, pipelineId) => { var _a; return (_a = register_js_1.internal.pipelines.get((0, register_js_1.audienceKey)(audience))) === null || _a === void 0 ? void 0 : _a.delete(pipelineId); };
exports.detachPipeline = detachPipeline;
const togglePausePipeline = (audience, pipelineId, paused) => {
    const key = (0, register_js_1.audienceKey)(audience);
    const existing = register_js_1.internal.pipelines.get(key);
    const p = existing === null || existing === void 0 ? void 0 : existing.get(pipelineId);
    existing &&
        p &&
        register_js_1.internal.pipelines.set(key, existing.set(p.id, Object.assign(Object.assign({}, p), { paused })));
};
exports.togglePausePipeline = togglePausePipeline;
const tailPipeline = (audience, { request }) => {
    var _a, _b;
    console.debug("received a tail command for audience", audience);
    if (!request) {
        console.debug("no tail reqeuest details specified, skipping");
        return;
    }
    switch (request.type) {
        case sp_common_1.TailRequestType.START: {
            console.debug("received a START tail: adding entry to audiences for tail id", audience);
            // Create inner map if it doesn't exist
            if (!register_js_1.internal.audiences.has((0, register_js_1.audienceKey)(audience))) {
                register_js_1.internal.audiences.set((0, register_js_1.audienceKey)(audience), {
                    audience,
                    tails: new Map(),
                });
            }
            // Add entry (@JH, OK if overwritten?)
            request.id &&
                ((_a = register_js_1.internal.audiences.get((0, register_js_1.audienceKey)(audience))) === null || _a === void 0 ? void 0 : _a.tails.set(request.id, {
                    tail: request.type === sp_common_1.TailRequestType.START,
                    tailRequestId: request.id,
                }));
            break;
        }
        case sp_common_1.TailRequestType.STOP: {
            console.debug("received a STOP tail: removing entry from audiences for tail id", request.id);
            request.id &&
                ((_b = register_js_1.internal.audiences.get((0, register_js_1.audienceKey)(audience))) === null || _b === void 0 ? void 0 : _b.tails.delete(request.id));
            break;
        }
        default:
            console.error("unknown tail request type ", request.type);
            break;
    }
};
exports.tailPipeline = tailPipeline;
